/*
Fox flasher
Runs on bs_nfet ESC.
 */
 
#include <avr/io.h>
#include <avr/sleep.h>
#include "bs_nfet.h"

//some macros for IO port bit manipulation
#ifndef cbi
#define cbi(sfr, bit) (sfr&=~(_BV(bit)))
#endif
#ifndef sbi
#define sbi(sfr, bit) (sfr|=(_BV(bit)))
#endif 


long randNumber;
int battery = 0; 

void setup() {                
  analogReference(DEFAULT); //setup Vcc (5V) reference
  //initialise port regs
  PORTB=INIT_PB;
  DDRB=DIR_PB;
  PORTC=INIT_PC;
  DDRC=DIR_PC;
  PORTD=INIT_PD;
  DDRD=DIR_PD;
  //see the pseudo random number generator
  randomSeed(analogRead(VIN));
}

void loop() {
  //LEDs off
  cbi(AnFET_port,AnFET);
  cbi(BnFET_port,BnFET);
  cbi(CnFET_port,CnFET);
  delayMicroseconds(20); //protect against shoot through
  cbi(ApFET_port,ApFET);
  cbi(BpFET_port,BpFET);
  cbi(CpFET_port,CpFET);

  //check daylight
  if(bit_is_clear(rcp_in_port,rcp_in)){
    delay(1000);
    return;
    };

  //check battery
  battery=analogRead(VIN);    
  if(battery<400){ //22k,5k1 divider 10.4*5.1/(22+5.1)/5*1023=400
    //sleep to save power
    set_sleep_mode(SLEEP_MODE_PWR_DOWN);
    cli();
    sleep_enable();
    sei();
    sleep_cpu();
    //needs reset to recover (cycle the power)
    };

  randNumber=random(300,2000);
  delay(randNumber);

  //LED on
  switch(randNumber%3){
    case 0:
    sbi(ApFET_port,ApFET);
    delayMicroseconds(20); //protect against shoot through
    sbi(AnFET_port,AnFET);
    break;
    case 1:
    sbi(BpFET_port,BpFET);
    delayMicroseconds(20); //protect against shoot through
    sbi(BnFET_port,BnFET);
    break;
    case 2:
    sbi(CpFET_port,CpFET);
    delayMicroseconds(20); //protect against shoot through
    sbi(CnFET_port,CnFET);
    break;
    }    
  randNumber=random(50,300);
  delay(randNumber);
}
